% Based on the following paper: Hong, S. (1999). Generating correlation matrices with model error for simulation studies in factor analysis: A combination of the Tucker-Koopman-Linn model and Wijsmans algorithm. Behavior Research Methods, Instruments, & Computers, 31, 727730.

function [OBS,W]=Hong(r,k,n,V,mfv,ifc,ifcw,eps,q)

% q minor factors
% r common (major) factors
% k measured variables
% n sample size

%V=    k*r             True loadings / major factor loading matrix
%D=    k*k             Diagonal matrix of unique factor variances
%W=    k*q             Minor factor loading matrix
%J=    k*(r+q)         Super loading matrix with columns made up of V and W
%B=    (r+q)*(r+q)     Super matrix of factor correlations
%S=    r*r             Correlations among major factors
%H=    r*q             Correlations among major and minor factors
%L=    q*q             Correlations among minor factors
%P=    k*k             Population correlation matrix of order k 
%X=    n*k             Random matrix with mean 0 and variance 1
%F=    k*k             Principal component matrix of P
%U=    k*k             Matrix with columns  from the k eigenvectors of P
%D2=    k*k            Diagonal matrix with the k eigenvalues as elements
%OBS=  n*k             Multivariate normal population with 0 means and 1 SDs

if ifcw==1       % correlation between all factors
    S=diag(1*ones(r,1));S(S==0)=ifc;
    H=ifc*ones(r,q);
    L=diag(ones(q,1));L(L==0)=ifc;
elseif ifcw==2   % correlation between F1&F2 factors only
    S=diag(1*ones(r,1));S(1,2)=ifc;S(2,1)=ifc;
    H=0*ones(r,q);
    L=diag(ones(q,1));L(L==0)=0;
end
B= [S H;H' L];

W=zeros(k,q);
if mfv>0   % if model error
    SD=1;
    for i=1:q
        W(1:k,i)=randn(1,k).*SD;
        SD=SD*(1-eps);
    end
    for i=1:k
        W(i,:)=W(i,:)./  sqrt((1/mfv*sum(W(i,:).^2)));
    end
end

J=[V W];
temp=J*B*J';
D=eye(k)-diag(diag(temp));
P=temp+D;
X=randn(n,k);
[U,D2]=eig(P);
F=U*D2^.5;
OBS=X*F';
